/* $Id: crires_dfs.c,v 1.29 2012-10-09 08:18:01 yjung Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: yjung $
 * $Date: 2012-10-09 08:18:01 $
 * $Revision: 1.29 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
   								Includes
 -----------------------------------------------------------------------------*/

#include <string.h>
#include <math.h>

#include <cpl.h>

#include "crires_dfs.h"

/*----------------------------------------------------------------------------*/
/**
 * @defgroup crires_dfs  DFS related functions
 *
 * TBD
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Set the group as RAW or CALIB in a frameset
  @param    set         the input frameset
  @param    recipe_name the recipe name
  @return   0 if ok, -1 in error case
 */
/*----------------------------------------------------------------------------*/
int crires_dfs_set_groups(
        cpl_frameset    *   set,
        const char      *   recipe_name)
{
    cpl_frame   *   cur_frame ;
    const char  *   tag ;
    int             nframes ;
    int             i ;

    /* Check entries */
    if (set == NULL) return -1 ; 
    
    /* Initialize */
    nframes = cpl_frameset_get_size(set) ;

    /* Loop on frames */
    for (i=0 ; i<nframes ; i++) {
        cur_frame = cpl_frameset_get_position(set, i) ;
        tag = cpl_frame_get_tag(cur_frame) ;
       
        /* RAW frames */
        if (!strcmp(tag, CRIRES_COMMAND_LINE) ||
                !strcmp(tag, CRIRES_UTIL_GENLINES_RAW) ||
                !strcmp(tag, CRIRES_UTIL_GENYPOS_RAW) ||
                !strcmp(tag, CRIRES_UTIL_GENCONFIG_RAW) ||
                !strcmp(tag, CRIRES_UTIL_GENSTD_RAW) ||
                !strcmp(tag, CRIRES_SPEC_JITTER_RAW) ||
                !strcmp(tag, CRIRES_SPEC_JITTER_J_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_OBJECT_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_OBJECT_RAW) ||
                !strcmp(tag, CRIRES_SPEC_GENERIC_OBJECT_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_SKY_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_SKY_RAW) ||
                !strcmp(tag, CRIRES_SPEC_GENERIC_SKY_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_J_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_J_RAW) ||
                !strcmp(tag, CRIRES_SPEC_ASTRO_RAW) ||
                !strcmp(tag, CRIRES_SPEC_DARK_RAW) ||
                !strcmp(tag, CRIRES_WIN_DARK_RAW) ||
                !strcmp(tag, CRIRES_SPEC_FLAT_RAW) ||
                !strcmp(tag, CRIRES_WIN_FLAT_RAW) ||
                !strcmp(tag, CRIRES_SPEC_WAVECAL_SKY_RAW) ||
                !strcmp(tag, CRIRES_SPEC_WAVECAL_LAMP_RAW) ||
                !strcmp(tag, CRIRES_SPEC_WAVECAL_ABS_RAW) ||
                !strcmp(tag, CRIRES_SPEC_JITTER_STD_RAW) ||
                !strcmp(tag, CRIRES_SPEC_JITTER_J_STD_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_STD_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_STD_RAW) ||
                !strcmp(tag, CRIRES_SPEC_NODDING_J_STD_RAW) ||
                !strcmp(tag, CRIRES_WIN_NODDING_J_STD_RAW) ||
                !strcmp(tag, CRIRES_MODEL_REFINE_RAW) ||
                !strcmp(tag, CRIRES_MODEL_WLMAP_RAW) ||
                !strcmp(tag, CRIRES_MODEL_FIX_RAW)) {
            cpl_frame_set_group(cur_frame, CPL_FRAME_GROUP_RAW) ;

            /* Check if the RAW file is valid for the current recipe */
            if (recipe_name != NULL) {
                if (!strcmp(recipe_name, "crires_model_fix") &&
                        strcmp(tag, CRIRES_MODEL_FIX_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_model_refine") &&
                        strcmp(tag, CRIRES_MODEL_REFINE_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_model_wlmap") &&
                        strcmp(tag, CRIRES_MODEL_WLMAP_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_spec_astro") &&
                        strcmp(tag, CRIRES_SPEC_ASTRO_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_spec_dark") &&
                        strcmp(tag, CRIRES_SPEC_DARK_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_spec_flat") &&
                        strcmp(tag, CRIRES_SPEC_FLAT_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_spec_jitter") &&
                        strcmp(tag, CRIRES_SPEC_JITTER_RAW) &&
                        strcmp(tag, CRIRES_SPEC_JITTER_J_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_OBJECT_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_SKY_RAW) &&
                        strcmp(tag, CRIRES_SPEC_GENERIC_OBJECT_RAW) &&
                        strcmp(tag, CRIRES_SPEC_GENERIC_SKY_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_J_RAW) &&
                        strcmp(tag, CRIRES_SPEC_JITTER_STD_RAW) &&
                        strcmp(tag, CRIRES_SPEC_JITTER_J_STD_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_STD_RAW) &&
                        strcmp(tag, CRIRES_SPEC_NODDING_J_STD_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_win_jitter") &&
                        strcmp(tag, CRIRES_WIN_NODDING_OBJECT_RAW) &&
                        strcmp(tag, CRIRES_WIN_NODDING_SKY_RAW) &&
                        strcmp(tag, CRIRES_WIN_NODDING_RAW) &&
                        strcmp(tag, CRIRES_WIN_NODDING_J_RAW) &&
                        strcmp(tag, CRIRES_WIN_NODDING_STD_RAW) &&
                        strcmp(tag, CRIRES_WIN_NODDING_J_STD_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_win_dark") &&
                        strcmp(tag, CRIRES_WIN_DARK_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_win_flat") &&
                        strcmp(tag, CRIRES_WIN_FLAT_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
                if (!strcmp(recipe_name, "crires_spec_wavecal") &&
                        strcmp(tag, CRIRES_SPEC_WAVECAL_SKY_RAW) &&
                        strcmp(tag, CRIRES_SPEC_WAVECAL_LAMP_RAW) &&
                        strcmp(tag, CRIRES_SPEC_WAVECAL_ABS_RAW)) {
                    cpl_msg_error(__func__, 
                            "The tag %s is invalid for this recipe", tag) ;
                    return -1 ;
                }
            }
        /* CALIB frames */
        } else if (!strcmp(tag, CRIRES_CALPRO_BPM) ||
                !strcmp(tag, CRIRES_CALPRO_BPM_WIN) ||
                !strcmp(tag, CRIRES_CALPRO_FLAT) ||
                !strcmp(tag, CRIRES_CALPRO_FLAT_WIN) ||
                !strcmp(tag, CRIRES_CALPRO_DARK) ||
                !strcmp(tag, CRIRES_CALPRO_DARK_WIN) ||
                !strcmp(tag, CRIRES_CALPRO_COEFFS_CUBE) ||
                !strcmp(tag, CRIRES_CALPRO_WAVE) ||
                !strcmp(tag, CRIRES_CALPRO_THAR_CAT) ||
                !strcmp(tag, CRIRES_CALPRO_N2O_CAT) ||
                !strcmp(tag, CRIRES_CALPRO_OH_CAT) ||
                !strcmp(tag, CRIRES_CALPRO_HITRAN_CAT) ||
                !strcmp(tag, CRIRES_CALPRO_STD_PHOTOFLUX) ||
                !strcmp(tag, CRIRES_CALPRO_THAR_POS) ||
                !strcmp(tag, CRIRES_CALPRO_MODEL_WAVEEG) ||
                !strcmp(tag, CRIRES_CALPRO_MODEL_CONFIG) ||
                !strcmp(tag, CRIRES_CALPRO_MODEL_REFINE_CONF) ||
                !strcmp(tag, CRIRES_CALPRO_MODEL_FIX_TAB))
            cpl_frame_set_group(cur_frame, CPL_FRAME_GROUP_CALIB) ;

    }
    return 0 ;
}

/**@}*/
