/* $Id: crires_model_metric.c,v 1.19 2012-01-12 17:58:17 bristowp Exp $
 *
 * This file is part of the CRIRES Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: bristowp $
 * $Date: 2012-01-12 17:58:17 $
 * $Revision: 1.19 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                                                   Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "crires_model_kernel.h"
#include "crires_model_io.h"
#include "crires_model_metric.h"

#include <stdio.h>
#include <math.h>
#include <float.h>
#include "crires_model_sa.h"

#include <stdlib.h>

#include "crires_model_cputime.h"


/* #include <rands.h> */
#include "crires_model_r250.h"

#include "irplib_utils.h"
#include "crires_load.h"
#include "crires_utils.h"
#include "crires_pfits.h"
#include "crires_dfs.h"

#ifdef _R250_H_
#define uniform(a,b)    ( a + (b - a) * dr250() )
#endif

/* #define MELT_ONLY  */
/* #define EQUIL_ONLY        */

#define XBIG_VAL 10000000.0
#define BIG_VAL   9900000.0

#define MAXIT 160000

/* for conversion from degree to radians */
/* static const DOUBLE DEG2RAD=M_PI/180.0; */
  
int n = 0;
int *chip, *x, *y;
struct crires *local_p_crires;
int local_nparam;
double * local_p_abest;
double * local_p_amin;
double * local_p_amax;
int * local_p_aname;
ann_all_par *local_p_all_par;

DOUBLE p_obsx[17000],p_obsy[17000],p_obsf[17000],p_penc[17000],p_genc[17000],p_slitw[17000],*p_wl; 
DOUBLE ** znse;
int p_obschip[17000], p_obsmode[17000], sp_array[17000], sindex[17000]; 
int size, mm;

/* the cost function
float  crires_model_energy (double *a);
void crires_model_assign_crires(int loc, double val);
void crires_model_output_cfg(struct crires *p_crires);
void output_data(double *);*/
 
cpl_table * crires_model_anneal_comp(ann_all_par *p_all_par,
                             int nparam,double * abest,
                             double * amin,
                             double * amax,
                             int * aname,
                             struct crires *p_crires,
                             int DS_size,
                             coord *msp_coord,
                             DOUBLE *p_wlarray,
                             DOUBLE ** znse_ref,
                             int morder)
{
  int ii;
  float z, t0, t;
  cpl_table * conf_tab ;
  double *a;

  fprintf(stderr,"%d points read for analysis\n", DS_size);

  report_cpu( stderr, NULL );
  printf("%d nparam \n",nparam);
  if ( SAInit(crires_model_energy,nparam) == 0 ) {
    cpl_msg_error(__func__, "trouble initializing in SAInit");
    return NULL;
  }

  local_p_abest=abest;
  local_p_amax=amax;
  local_p_amin=amin;
  local_p_aname=aname;
  local_nparam=nparam;
  local_p_all_par=p_all_par;
  local_p_crires=p_crires;
  for (ii=0;ii<DS_size;ii++) {
    p_obsx[ii]=msp_coord[ii].x;
    p_obsy[ii]=msp_coord[ii].y;
    p_obschip[ii]=msp_coord[ii].chip;
    p_obsf[ii]=msp_coord[ii].flux;
    sp_array[ii]=msp_coord[ii].slit_pos;
    p_obsmode[ii]=msp_coord[ii].mode;
    /*convert encoder values to floats here so that it
      does not have to been done every time the angle is
      computed*/
    p_penc[ii]=(double)(msp_coord[ii].penc);
    p_genc[ii]=(double)(msp_coord[ii].genc);
    sindex[ii]=msp_coord[ii].counter;
    p_slitw[ii]=msp_coord[ii].slitw;
    printf("%d obsx: %lf obsy:%lf, obschip: %d order: %d sp: %d penc: %lf genc: %lf\n",
           ii, p_obsx[ii],p_obsy[ii],p_obschip[ii],p_obsmode[ii],
           sp_array[ii],p_penc[ii],p_genc[ii]);
  }
  p_wl=p_wlarray;        
  size = DS_size; 
  znse=znse_ref;

  a=(DOUBLE *)cpl_malloc(nparam*sizeof(DOUBLE));

  mm=morder;
  for (ii=0; ii<local_nparam ; ii++) {
    a[ii]=local_p_abest[ii];
  }
  printf("bef ener %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  z = crires_model_energy(a);
        
  printf("bef init %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);
  SAinitial( a );
  SABoltzmann( 0.5 );
  printf("aft boltz %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

/*        SAjump( 150.0 ); */
        
  fprintf(stderr,"Boltzman constant: %f\tlearning rate:%f\n",
          SABoltzmann( NO_VALUE ), SAlearning_rate( NO_VALUE ) );

  fprintf(stderr,"Initial values: ");
  for (ii = 0; ii < local_nparam; ii++)
    fprintf(stderr,"%g\t", a[ii]);
  fprintf(stderr,"\t(energy = %g)\n", z);

  SAmelt( NO_VALUE );                /* melt the system */
  printf("aft melt %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  SAcurrent( a );
  printf("aft current %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  z = crires_model_energy(a);

  printf("aft energy %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  fprintf(stderr,"melt values: ");
  for (ii = 0; ii < local_nparam; ii++)
    fprintf(stderr,"%g\t", a[ii]);
  fprintf(stderr,"\t(energy = %g)\n", z);

  t0 = SAtemperature( NO_VALUE );
  printf("aft temp %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  /* make it a bit warmer than melting temperature */
  t0 = 1.2 * SAtemperature( NO_VALUE );

  SAtemperature( t0 );
  printf("aft temp t0 %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  ii=MAXIT;
  printf("%d\n",ii);
  t = SAanneal( MAXIT );
  printf("aft anneal %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);
  SAcurrent( a );
  printf("aft current %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf \n",
         a[0], a[1],a[2], a[3],a[4],a[5], a[6],a[7], a[8],a[9]);
  z = crires_model_energy(a);
  printf("aft energy %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf \n",
         a[0], a[1],a[2], a[3],a[4],a[5], a[6],a[7], a[8],a[9]);

  SAfree();

  fprintf(stderr,"Initial temperature: %f\tfinal temperature: %f\n", t0, t);
  fprintf(stderr,"Estimated minimum at: ");
  for (ii = 0; ii < local_nparam; ii++)
    fprintf(stderr,"%g\t", a[ii]);
  fprintf(stderr,"\t(energy = %g)\n", z);


  /* Below is commented out. To be honest its not clear to me what SAoptimum was ever supposed to do, however it is clear that it changes a[ii] from the values that gave the best solution during the anneal, so that the output solution is then different (and usually worse). Better to leave it out then */
  /*SAoptimum( a );
  printf("aft opt %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);
  z = crires_model_energy(a);
  printf("aft energy %lf %lf %lf %lf %lf \n",a[0], a[1],a[2], a[3],a[4]);

  fprintf(stderr,"Best minimum at: ");
  for (ii = 0; ii < local_nparam; ii++)
    fprintf(stderr,"%g\t", a[ii]);
    fprintf(stderr,"\t(energy = %g)\n", z);*/

  conf_tab = crires_model_io_output_cfg(local_p_crires);
  crires_model_output_data(a); 

  for (ii = 0; ii < local_nparam; ii++)
  {
    abest[ii]=local_p_abest[ii];
  }

  cpl_free(a);

  report_cpu( stderr, NULL );
  return conf_tab ;
}

void crires_model_output_data(double *a)
{
  static int counter;
  int ii;
  double penc, genc,penc2, genc2;
  double quad_diff,b; 
  
  for (ii=0; ii<local_nparam; ii++) {
    //crires_model_assign_crires(local_p_aname[ii],local_p_abest[ii]+((local_p_amax[ii]-local_p_amin[ii])/2.0)*a[ii]);
    //printf("%lf ",a[ii]);
    b=a[ii];
    }


  printf("\n");
  /*printf(" %4.4lf %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f\n",
         local_p_crires->temper,
         local_p_crires->es_x, local_p_crires->mues/DEG2RAD,
         local_p_crires->,local_p_crires->nues/DEG2RAD ,
         local_p_crires->taues/DEG2RAD,
         local_p_crires->,local_p_crires->mup1/DEG2RAD,
         local_p_crires->,local_p_crires->nup1/DEG2RAD,
         local_p_crires->,local_p_crires->taup1/DEG2RAD,
         local_p_crires->,local_p_crires->mug/DEG2RAD,local_p_crires->sg,
         local_p_crires->pix, local_p_crires->fcp,
         local_p_crires->slity,
         local_p_crires->cam_dis,
         local_p_crires->fk,
         local_p_crires->taud/DEG2RAD,
         local_p_crires->det_array_x,
         local_p_crires->det_array_y);*/

  if (crires_model_init(local_p_crires)!=0) {
    cpl_msg_error(__func__, "Failed to initialise model" );
  }

  quad_diff=0;
  for (ii = 0; ii < size; ii++) {
    local_p_crires->es_x=local_p_crires->slit[sp_array[ii]];

    if (p_penc[ii]<1.0) {
      local_p_crires->mup1=local_p_crires->prism[(int)(p_genc[ii])];
      local_p_crires->mug=local_p_crires->grat[(int)(p_genc[ii])];
    }
    else {
      penc=p_penc[ii]-36000.0;
      penc2=penc*penc;
      local_p_crires->mup1=local_p_crires->penc_co1+
	local_p_crires->penc_co2*penc+
	local_p_crires->penc_co3*penc2;
      genc=p_genc[ii]-200000.0;
      genc2=genc*genc;
      local_p_crires->mug=local_p_crires->genc_co1+
	local_p_crires->genc_co2*genc+
	local_p_crires->genc_co3*genc2;
    }
    if (p_obsmode[ii]>=1) {
      mm=local_p_crires->ord[p_obsmode[ii]];
    }
    else {
      mm=-p_obsmode[ii];
    }
    local_p_crires->slitw=p_slitw[ii];
    if (crires_model_init(local_p_crires)!=0) {
      cpl_msg_error(__func__, "Failed to initialise model" );
    }

    crires_model_eval(p_wl[ii],mm,znse,local_p_crires);
    crires_model_detpix(local_p_crires);
    if (p_obschip[ii]==local_p_crires->chippix[0]) {
      quad_diff+=
        ((p_obsx[ii]-local_p_crires->xpospix)*
         (p_obsx[ii]-local_p_crires->xpospix)+
         (p_obsy[ii]-local_p_crires->ypospix)*
         (p_obsy[ii]-local_p_crires->ypospix));
      printf("%d  %lf %lf  %lf %lf  %lf %lf  %d %d  %lf  %lf %lf %lf %lf %lf %lf %lf %lf\n",
             ii,p_obsx[ii]-local_p_crires->xpospix,
             p_obsy[ii]-local_p_crires->ypospix,p_obsx[ii],
             local_p_crires->xpospix,p_obsy[ii],local_p_crires->ypospix,
             p_obschip[ii],p_obsmode[ii], p_obsf[ii], p_wl[ii]*1000.0,
	     p_penc[ii], local_p_crires->mup1, p_genc[ii],local_p_crires->mug,
	     local_p_crires->grat_alpha,local_p_crires->grat_beta,local_p_crires->slitw);
    }
/*     else { */
/*       printf("%d  %d %lf  %lf %lf  %lf %lf  %d %d  %lf  %lf %lf %lf\n", */
/*              ii,local_p_crires->chippix[0], */
/*              0.0,p_obsx[ii], */
/*              local_p_crires->xpospix,p_obsy[ii],local_p_crires->ypospix, */
/*              p_obschip[ii],p_obsmode[ii], p_obsf[ii], p_wl[ii]*1000.0, p_penc[ii], p_genc[ii]); */
/*     } */
    counter++;
  }
  //  printf("%lf\n",quad_diff);
}

/* evaluate the sum squared error, the "energy" */
float crires_model_energy(double *a) {
  int flux_wt=0;
  //  FILE * plot; 
  static int counter;
  static int first_time;
  static float bestyet;
  double newval[150];
  //float ** plotout=crires_model_alloc2Darray_f(size,6);
 
  int ii;
  float sum_nw = 0.0, val, val_slit;
  float sum_fw = 0.0, val_fw;
  float sum_x = 0.0, val_x;
  float sum_y = 0.0, val_y;
  float sum = 0.0;
  double penc, genc,penc2, genc2;

  //  plot=fopen("plot.dat","a+");    

  for (ii=0; ii<local_nparam; ii++) {
    if(first_time >0) {
      newval[ii]=local_p_abest[ii]+((local_p_amax[ii]-local_p_amin[ii])/2.0)*a[ii];
    }
    else {
      newval[ii]=local_p_abest[ii];
    }
    //printf("\n %d %lf %lf %lf  %lf %lf\n",ii, newval,local_p_amax[ii],local_p_amin[ii],a[ii], local_p_abest[ii]);
    if (newval[ii]>local_p_amax[ii] || newval[ii]<local_p_amin[ii]) {
      sum_nw=HUGE_VAL;
      sum_fw=HUGE_VAL;
    }
    crires_model_assign_crires(local_p_aname[ii],newval[ii]);
    //if (sum!=HUGE_VAL) printf("%s %lf  ", (local_p_all_par+local_p_aname[ii])->name,newval);
  }
  if (crires_model_init(local_p_crires)!=0) {
    cpl_msg_error(__func__, "Failed to initialise model" );
    return -1.0;
  }

  for (ii = 0; ii < size; ii++) {
    if (sum_nw<HUGE_VAL) {
      local_p_crires->es_x=local_p_crires->slit[sp_array[ii]];
      if (p_penc[ii]<1.0) {
	local_p_crires->mup1=local_p_crires->prism[(int)(p_genc[ii])];
	local_p_crires->mug=local_p_crires->grat[(int)(p_genc[ii])];
      }
      else {
	penc=p_penc[ii]-36000.0;
	penc2=penc*penc;
	local_p_crires->mup1=local_p_crires->penc_co1+
	  local_p_crires->penc_co2*penc+
	  local_p_crires->penc_co3*penc2;
	genc=p_genc[ii]-200000.0;
	genc2=genc*genc;
	local_p_crires->mug=local_p_crires->genc_co1+
	  local_p_crires->genc_co2*genc+
	  local_p_crires->genc_co3*genc2;
      }
      if (p_obsmode[ii]>=1) {
	mm=local_p_crires->ord[p_obsmode[ii]];
      }
      else {
	mm=-p_obsmode[ii];
      }

      local_p_crires->slitw=p_slitw[ii];
      if (crires_model_init(local_p_crires)!=0) {
	cpl_msg_error(__func__, "Failed to initialise model" );
	return -1.0;
      }

      crires_model_eval(p_wl[ii],mm,znse,local_p_crires);
      crires_model_detpix(local_p_crires);
      if (p_obschip[ii]==local_p_crires->chippix[0]) {
        val_x=p_obsx[ii]-local_p_crires->xpospix;
	if (val_x<0.0) val_x=-val_x;
        val_y=p_obsy[ii]-local_p_crires->ypospix;
	if (val_y<0.0) val_y=-val_y;
	val=val_x*val_x+val_y*val_y;
        val_fw=val*p_obsf[ii];
	if (local_p_crires->onslit==0) val=val*2.0;
        //printf("%d   %f    %lf    %lf  \n",ii,val,p_obsx[ii]-local_p_crires->xpospix,p_obsy[ii]-local_p_crires->ypospix);
        //printf("%4.4lf   %4.4lf\n", local_p_crires->xpospix, local_p_crires->ypospix);
        /*plotout[ii][0]=(float)(local_p_crires->chippix[0]);
        plotout[ii][1]=sqrt(val);
        plotout[ii][2]=p_obsx[ii]-local_p_crires->xpospix;
        plotout[ii][3]=p_obsy[ii]-local_p_crires->ypospix;
        plotout[ii][4]=local_p_crires->ypospix;
        plotout[ii][5]=local_p_crires->xpospix+local_p_crires->chipx[local_p_crires->chippix[0]]/local_p_crires->pix;*/
      }
      else if (local_p_crires->chippix[0]==-1) {
	val=XBIG_VAL;
	val_x=XBIG_VAL;
	val_y=XBIG_VAL;
	val_fw=XBIG_VAL;
      }
      else {
        val=BIG_VAL;
        val_x=BIG_VAL;
        val_y=BIG_VAL;
        val_fw=BIG_VAL;
      }
      val_slit=((local_p_crires->slity-local_p_crires->yis)*(local_p_crires->slity-local_p_crires->yis)+
		(p_obsy[ii]-local_p_crires->ypospix)*
		(p_obsy[ii]-local_p_crires->ypospix));

      sum_nw += val;
      sum_fw += val_fw;
      sum_x +=val_x;
      sum_y +=val_y;
      //printf("%d   %f    %lf    %f    %lf    %lf %d %d\n",ii,val,p_obsy[ii],local_p_crires->xdet,p_obsx[ii],local_p_crires->ydet,p_obschip[ii],local_p_crires->chippix[0]);
      /*if(first_time==0)
        printf("%4.4lf   %4.4lf   %4.4lf   %4.4lf \n",p_obsx[i],x,p_obsy[i],y);*/
      //printf("%f \n",val);
    }
  }
  if (flux_wt==1) {
    sum=sum_fw;
  }
  else if (flux_wt==2) {
    sum=sum_x*sum_x+sum_y*sum_y;
  }
  else {
    sum=sum_nw;
  }

  //if (sum!=HUGE_VAL) printf("%d %f %f %d %f \n",first_time, sqrt(sum/(float)(size)), sum,size,bestyet);
  if (sqrt(sum/size)<bestyet) {
    printf("it: %d nw %lf fw %lf \n",counter, sqrt(sum_nw/size), sqrt(sum_fw/size));
    printf("x %lf y %lf \n", sum_x/size, sum_y/size);
    for (ii=0; ii<local_nparam; ii++) {
      printf("%s  %f  %f %f  %f \n", (local_p_all_par+local_p_aname[ii])->name, newval[ii],local_p_amax[ii],local_p_amin[ii],(newval[ii]-local_p_amin[ii])/(local_p_amax[ii]-local_p_amin[ii]));
    }
    //    crires_model_io_output_cfg_txt(local_p_crires);
    bestyet=sqrt(sum/size);
    //}
  }
  counter++;
  if(first_time==0) {
    first_time++;
    bestyet=1000000;
  }
  //  fclose(plot); 
  return sum;
  //  crires_model_free2Darray_f(plotout,size);
        /*}*/
}

void crires_model_assign_crires(int loc, double val)
{
  int compa,kk,indlen;
  char tempstr[10];
  compa=strncmp((local_p_all_par+loc)->name,"temper",6);
  if (compa==0) local_p_crires->temper=val;
  compa=strncmp((local_p_all_par+loc)->name,"es_xf",5);
  if (compa==0) local_p_crires->es_xf=val;
  compa=strncmp((local_p_all_par+loc)->name,"es_x_off",8);
  if (compa==0) local_p_crires->es_x_off=val;
  compa=strncmp((local_p_all_par+loc)->name,"es_y",4);
  if (compa==0) local_p_crires->es_y=val;
  compa=strncmp((local_p_all_par+loc)->name,"mues",4);
  if (compa==0) local_p_crires->mues=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nues",4);
  if (compa==0) local_p_crires->nues=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"taues",5);
  if (compa==0) local_p_crires->taues=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"es_s",4);
  if (compa==0) local_p_crires->es_s=val;
  compa=strncmp((local_p_all_par+loc)->name,"es_w",4);
  if (compa==0) local_p_crires->es_w=val;
  compa=strncmp((local_p_all_par+loc)->name,"fcp",3);
  if (compa==0) local_p_crires->fcp=val;
  compa=strncmp((local_p_all_par+loc)->name,"mup1",4);
  if (compa==0) local_p_crires->mup1=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_mup",8);
  if (compa==0) local_p_crires->axis_mup=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"penc_co1",8);
  if (compa==0) local_p_crires->penc_co1=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"penc_co2",8);
  if (compa==0) local_p_crires->penc_co2=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"penc_co3",8);
  if (compa==0) local_p_crires->penc_co3=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nup1",4);
  if (compa==0) local_p_crires->nup1=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"taup1",5);
  if (compa==0) local_p_crires->taup1=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_nup",8);
  if (compa==0) local_p_crires->axis_nup=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_taup",9);
  if (compa==0) local_p_crires->axis_taup=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"mup2",4);
  if (compa==0) local_p_crires->mup2=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nup2",4);
  if (compa==0) local_p_crires->nup2=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"taup2",5);
  if (compa==0) local_p_crires->taup2=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"rind_delta",10);
  if (compa==0) local_p_crires->rind_delta=val;
  compa=strncmp((local_p_all_par+loc)->name,"muis",4);
  if (compa==0) local_p_crires->muis=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nuis",4);
  if (compa==0) local_p_crires->nuis=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"tauis",5);
  if (compa==0) local_p_crires->tauis=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"flipisx",7);
  if (compa==0) local_p_crires->flipisx=val;
  compa=strncmp((local_p_all_par+loc)->name,"flipisy",7);
  if (compa==0) local_p_crires->flipisy=val;
  compa=strncmp((local_p_all_par+loc)->name,"fis",3);
  if (compa==0) local_p_crires->fis=val;
  compa=strncmp((local_p_all_par+loc)->name,"slitx",5);
  if (compa==0) local_p_crires->slitx=val;
  compa=strncmp((local_p_all_par+loc)->name,"slity",5);
  if (compa==0) local_p_crires->slity=val;
  compa=strncmp((local_p_all_par+loc)->name,"slitrot",7);
  if (compa==0) local_p_crires->slitrot=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"slitw",5);
  if (compa==0) local_p_crires->slitw=val;
  compa=strncmp((local_p_all_par+loc)->name,"slith",5);
  if (compa==0) local_p_crires->slith=val;
  compa=strncmp((local_p_all_par+loc)->name,"mug",3);
  if (compa==0) local_p_crires->mug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_mug",8);
  if (compa==0) local_p_crires->axis_mug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"genc_co1",8);
  if (compa==0) local_p_crires->genc_co1=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"genc_co2",8);
  if (compa==0) local_p_crires->genc_co2=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"genc_co3",8);
  if (compa==0) local_p_crires->genc_co3=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_nug",8);
  if (compa==0) local_p_crires->axis_nug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"axis_taug",9);
  if (compa==0) local_p_crires->axis_taug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nug",3);
  if (compa==0) local_p_crires->nug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"taug",4);
  if (compa==0) local_p_crires->taug=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"sg",2);
  if (compa==0) local_p_crires->sg=val;
  compa=strncmp((local_p_all_par+loc)->name,"cam_dis",7);
  if (compa==0) local_p_crires->cam_dis=val;
  compa=strncmp((local_p_all_par+loc)->name,"muk",3);
  if (compa==0) local_p_crires->muk=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nuk",3);
  if (compa==0) local_p_crires->nuk=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"tauk",4);
  if (compa==0) local_p_crires->tauk=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"fk",2);
  if (compa==0) local_p_crires->fk=val;
  compa=strncmp((local_p_all_par+loc)->name,"mud",3);
  if (compa==0) local_p_crires->mud=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"nud",3);
  if (compa==0) local_p_crires->nud=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"taud",4);
  if (compa==0) local_p_crires->taud=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"pix",3);
  if (compa==0) local_p_crires->pix=val;

  compa=strncmp((local_p_all_par+loc)->name,"chipx[0]",8);
  if (compa==0) local_p_crires->chipx[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chipy[0]",8);
  if (compa==0) local_p_crires->chipy[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chiprot[0]",10);
  if (compa==0) local_p_crires->chiprot[0]=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"chipx[1]",8);
  if (compa==0) local_p_crires->chipx[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chipy[1]",8);
  if (compa==0) local_p_crires->chipy[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chiprot[1]",10);
  if (compa==0) local_p_crires->chiprot[1]=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"chipx[2]",8);
  if (compa==0) local_p_crires->chipx[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chipy[2]",8);
  if (compa==0) local_p_crires->chipy[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chiprot[2]",10);
  if (compa==0) local_p_crires->chiprot[2]=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"chipx[3]",8);
  if (compa==0) local_p_crires->chipx[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chipy[3]",8);
  if (compa==0) local_p_crires->chipy[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"chiprot[3]",10);
  if (compa==0) local_p_crires->chiprot[3]=val*DEG2RAD;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xx[0]",10);
  if (compa==0) local_p_crires->pc_x_xx[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_x1[0]",10);
  if (compa==0) local_p_crires->pc_x_x1[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_yy[0]",10);
  if (compa==0) local_p_crires->pc_x_yy[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_y1[0]",10);
  if (compa==0) local_p_crires->pc_x_y1[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xy[0]",10);
  if (compa==0) local_p_crires->pc_x_xy[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xx[0]",10);
  if (compa==0) local_p_crires->pc_y_xx[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_x1[0]",10);
  if (compa==0) local_p_crires->pc_y_x1[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_yy[0]",10);
  if (compa==0) local_p_crires->pc_y_yy[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_y1[0]",10);
  if (compa==0) local_p_crires->pc_y_y1[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xy[0]",10);
  if (compa==0) local_p_crires->pc_y_xy[0]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xx[1]",10);
  if (compa==0) local_p_crires->pc_x_xx[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_x1[1]",10);
  if (compa==0) local_p_crires->pc_x_x1[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_yy[1]",10);
  if (compa==0) local_p_crires->pc_x_yy[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_y1[1]",10);
  if (compa==0) local_p_crires->pc_x_y1[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xy[1]",10);
  if (compa==0) local_p_crires->pc_x_xy[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xx[1]",10);
  if (compa==0) local_p_crires->pc_y_xx[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_x1[1]",10);
  if (compa==0) local_p_crires->pc_y_x1[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_yy[1]",10);
  if (compa==0) local_p_crires->pc_y_yy[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_y1[1]",10);
  if (compa==0) local_p_crires->pc_y_y1[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xy[1]",10);
  if (compa==0) local_p_crires->pc_y_xy[1]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xx[2]",10);
  if (compa==0) local_p_crires->pc_x_xx[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_x1[2]",10);
  if (compa==0) local_p_crires->pc_x_x1[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_yy[2]",10);
  if (compa==0) local_p_crires->pc_x_yy[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_y1[2]",10);
  if (compa==0) local_p_crires->pc_x_y1[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xy[2]",10);
  if (compa==0) local_p_crires->pc_x_xy[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xx[2]",10);
  if (compa==0) local_p_crires->pc_y_xx[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_x1[2]",10);
  if (compa==0) local_p_crires->pc_y_x1[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_yy[2]",10);
  if (compa==0) local_p_crires->pc_y_yy[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_y1[2]",10);
  if (compa==0) local_p_crires->pc_y_y1[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xy[2]",10);
  if (compa==0) local_p_crires->pc_y_xy[2]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xx[3]",10);
  if (compa==0) local_p_crires->pc_x_xx[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_x1[3]",10);
  if (compa==0) local_p_crires->pc_x_x1[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_yy[3]",10);
  if (compa==0) local_p_crires->pc_x_yy[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_y1[3]",10);
  if (compa==0) local_p_crires->pc_x_y1[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_x_xy[3]",10);
  if (compa==0) local_p_crires->pc_x_xy[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xx[3]",10);
  if (compa==0) local_p_crires->pc_y_xx[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_x1[3]",10);
  if (compa==0) local_p_crires->pc_y_x1[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_yy[3]",10);
  if (compa==0) local_p_crires->pc_y_yy[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_y1[3]",10);
  if (compa==0) local_p_crires->pc_y_y1[3]=val;
  compa=strncmp((local_p_all_par+loc)->name,"pc_y_xy[3]",10);
  if (compa==0) local_p_crires->pc_y_xy[3]=val;

  compa=strncmp((local_p_all_par+loc)->name,"det_array_x",11);
  if (compa==0) local_p_crires->det_array_x=val;
  compa=strncmp((local_p_all_par+loc)->name,"det_array_y",11);
  if (compa==0) local_p_crires->det_array_y=val;

  for (kk=1;kk<60;kk++) {
    sprintf(tempstr,"prism[%d]",kk);
    indlen=strlen(tempstr);
    compa=strncmp((local_p_all_par+loc)->name,tempstr,indlen);
    if (compa==0) local_p_crires->prism[kk]=val*DEG2RAD;
  }
  for (kk=1;kk<60;kk++) {
    sprintf(tempstr,"grat[%d]",kk);
    indlen=strlen(tempstr);
    compa=strncmp((local_p_all_par+loc)->name,tempstr,indlen);
    if (compa==0) local_p_crires->grat[kk]=val*DEG2RAD;
  }
  for (kk=1;kk<10;kk++) {
    sprintf(tempstr,"slit[%d]",kk);
    indlen=strlen(tempstr);
    compa=strncmp((local_p_all_par+loc)->name,tempstr,indlen);
    if (compa==0) local_p_crires->slit[kk]=val;
  }

  compa=strncmp((local_p_all_par+loc)->name,"offx",4);
  if (compa==0) local_p_crires->offx=val;
  compa=strncmp((local_p_all_par+loc)->name,"offy",4);
  if (compa==0) local_p_crires->offy=val;
  compa=strncmp((local_p_all_par+loc)->name,"flipx",5);
  if (compa==0) local_p_crires->flipx=val;
  compa=strncmp((local_p_all_par+loc)->name,"flipy",5);
  if (compa==0) local_p_crires->flipy=val;

  compa=strncmp((local_p_all_par+loc)->name,"simpl",5);
  if (compa==0) local_p_crires->simpl=val;
  compa=strncmp((local_p_all_par+loc)->name,"nx",2);
  if (compa==0) local_p_crires->nx=val;
  compa=strncmp((local_p_all_par+loc)->name,"ny",2);
  if (compa==0) local_p_crires->ny=val;
  compa=strncmp((local_p_all_par+loc)->name,"uvab",4);
  if (compa==0) local_p_crires->uvab=val;
  compa=strncmp((local_p_all_par+loc)->name,"slitp",5);
  if (compa==0) local_p_crires->slitp=val;
}
