/* $Id: crires_model_randlcg.c,v 1.3 2008-05-15 17:23:11 bristowp Exp $
 *
 *Not sure about the copyright stuff here!
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: bristowp $
 * $Date: 2008-05-15 17:23:11 $
 * $Revision: 1.3 $
 * $Name: not supported by cvs2svn $
 */
/* rndlcg            Linear Congruential Method, the "minimal standard generator"
                     Park & Miller, 1988, Comm of the ACM, 31(10), pp. 1192-1201

*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
   								Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "crires_model_kernel.h"


/*----------------------------------------------------------------------------*/

#include <math.h>
#include <limits.h>
#include <crires_model_randlcg.h>

#define ALL_BITS     0xffffffff

static long int the_quotient  = LONG_MAX / 16807L;
static long int the_remainder = LONG_MAX % 16807L;

static long int seed_val = 1L;

long set_seed(long int sd)
{
        return seed_val = sd;
}

long get_seed(void)
{
        return seed_val;
}


unsigned long int randlcg(void)       /* returns a random unsigned integer */
{
        if ( seed_val <= the_quotient )
                seed_val = (seed_val * 16807L) % LONG_MAX;
        else
        {
                long int high_part = seed_val / the_quotient;
                long int low_part  = seed_val % the_quotient;

                long int test = 16807L * low_part - the_remainder * high_part;

                if ( test > 0 )
                        seed_val = test;
                else
                        seed_val = test + LONG_MAX;

        }

        return seed_val;
}





